@@ -1,14 +1,20 @@
 from abc import abstractmethod
 from collections.abc import AsyncIterator
-from typing import Optional
+from typing import TYPE_CHECKING, Optional
 
 from launart import Service
 
 from ..model import Event, Login
 from .route import RouterMixin
+from .utils import ctx
+
+if TYPE_CHECKING:
+    from . import Server
 
 
 class Adapter(Service, RouterMixin):
+    server: "Server"
+
     @abstractmethod
     def get_platform(self) -> str: ...
 
@@ -28,6 +34,10 @@ class Adapter(Service, RouterMixin):
     @abstractmethod
     async def download_uploaded(self, platform: str, self_id: str, path: str) -> bytes: ...
 
+    async def download_proxied(self, prefix: str, url: str) -> bytes:
+        async with self.server.session.get(url, ssl=ctx) as resp:
+            return await resp.read()
+
     @abstractmethod
     async def get_logins(self) -> list[Login]: ...
 
@@ -38,3 +48,6 @@ class Adapter(Service, RouterMixin):
     @property
     def id(self):
         return f"satori-python.adapter.{self.get_platform()}#{id(self)}"
+
+    def ensure_server(self, server: "Server"):
+        self.server = server
